//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     rule.h
// Purpose:    Rules
// Last Change:02.10.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2001 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	RULE_INCLUDED
#define RULE_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef BOOL_INCLUDED
#include "bool.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef RVARS_INCLUDED
#include "rvars.h"
#endif

#ifndef RLITS_INCLUDED
#include "rlits.h"
#endif

#ifndef RNEG_INCLUDED
#include "rneg.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// RULE_MAGIC: Magic number (identifies objects of this class for debugging).
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
static const long	RULE_MAGIC =	0x52554C00L;	// 'RUL\0'
#endif

//=============================================================================
// Class for literal (atomic formula) in a rule:
//=============================================================================

class rule_c {

	public:

//-----------------------------------------------------------------------------
// Methods:
//-----------------------------------------------------------------------------

	// Constructor:
	rule_c(rvars_t vars, rlits_t head, rlits_t body, rneg_t cond);

	// Destructor:
	~rule_c(void);

	// Return the set of variables in this rule:
	inline rvars_t	get_vars(void) const
		{ CHECK(valid(), "rule_c::get_vars: Invalid Object");
		  return(Vars); }

	// Return the set of head literals in this rule:
	inline rlits_t	get_head(void) const
		{ CHECK(valid(), "rule_c::get_head: Invalid Object");
		  return(Head); }

	// Return the set of positive body literals of this rule:
	inline rlits_t	get_body(void) const
		{ CHECK(valid(), "rule_c::get_pos_body: Invalid Object");
		  return(Body); }

	// Return the set of negative body literals in this rule:
	inline rneg_t	get_cond(void) const
		{ CHECK(valid(), "rule_c::get_cond: Invalid Object");
		  return(Cond); }

	// Print this rule:
	void print(void) const;

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity Check:
	public:
	bool_t	valid(void) const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "RULE_MAGIC"
#endif

//-----------------------------------------------------------------------------
// Private Members:
//-----------------------------------------------------------------------------

	private:

	// Set of variables in this rule:
	rvars_t Vars;

	// Set of head literals in this rule:
	rlits_t Head;

	// Set of positive body literals:
	rlits_t Body;

	// Set of Default Negations:
	rneg_t Cond;

//-----------------------------------------------------------------------------
// End of Class:
//-----------------------------------------------------------------------------

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef rule_c *rule_t;

//=============================================================================
// End of Include File:
//=============================================================================

#endif

